@@ -1,6 +1,6 @@
 /*
     ccid.c: CCID common code
-    Copyright (C) 2003   Ludovic Rousseau
+    Copyright (C) 2003-2005   Ludovic Rousseau
 
     This library is free software; you can redistribute it and/or
     modify it under the terms of the GNU Lesser General Public
@@ -18,7 +18,7 @@
 */
 
 /*
- * $Id: ccid.c,v 1.1.1.6 2005/03/15 18:55:25 rousseau Exp $
+ * $Id: ccid.c,v 1.2 2005-11-27 20:12:17 rousseau Exp $
  */
 
 #include <stdio.h>
@@ -112,7 +112,7 @@ int ccid_open_hack(unsigned int reader_index)
  *					ccid_error
  *
  ****************************************************************************/
-void ccid_error(int error, char *file, int line, char *function)
+void ccid_error(int error, char *file, int line, const char *function)
 {
 	char *text;
 	char var_text[20];
@@ -229,7 +229,7 @@ void ccid_error(int error, char *file, int line, char *function)
 				text = "Unknown CCID error";
 			break;
 	}
-	debug_msg("%s:%d:%s %s", file, line, function, text);
+	log_msg(PCSC_LOG_ERROR, "%s:%d:%s %s", file, line, function, text);
 
 } /* ccid_error */
 
