@@ -18,7 +18,7 @@
 */
 
 /*
- * $Id: ccid.h,v 1.1.1.5 2005/03/15 18:55:25 rousseau Exp $
+ * $Id: ccid.h,v 1.2 2005-11-27 20:12:17 rousseau Exp $
  */
 
 typedef struct
@@ -37,7 +37,7 @@ typedef struct
 	/*
 	 * Maximum message length
 	 */
-	int dwMaxCCIDMessageLength;
+	unsigned int dwMaxCCIDMessageLength;
 
 	/*
 	 * Maximum IFSD
@@ -62,7 +62,7 @@ typedef struct
 	/*
 	 * Max Data Rate
 	 */
-	int dwMaxDataRate;
+	unsigned int dwMaxDataRate;
 
 	/*
 	 * Number of available slots
@@ -74,6 +74,18 @@ typedef struct
 	 */
 	char bCurrentSlotIndex;
 
+	/*
+	 * The array of data rates supported by the reader
+	 */
+	unsigned int *arrayOfSupportedDataRates;
+	
+	/*
+	 * Read communication port timeout
+	 * value is seconds
+	 * this value can evolve dynamically if card request it (time processing).
+	 */
+	unsigned int readTimeout;
+
 } _ccid_descriptor;
 
 /* Features from dwFeatures */
@@ -109,7 +121,9 @@ typedef struct
 #define CARDMAN3121	0x076B3021
 #define LTC31		0x07830003
 #define SCR331DI	0x04E65111
+#define SCR331DINTTCOM	0x04E65120
 #define CHERRYXX33	0x046A0005
+#define CHERRYST2000	0x046A003E
 #define OZ776		0x0B977762
 #define SPR532		0x04E6E003
 
@@ -119,7 +133,7 @@ typedef struct
 
 
 int ccid_open_hack(unsigned int reader_index);
-void ccid_error(int error, char *file, int line, char *function);
+void ccid_error(int error, char *file, int line, const char *function);
 _ccid_descriptor *get_ccid_descriptor(unsigned int reader_index);
 
 /* convert a 4 byte integer in USB format into an int */
