@@ -18,22 +18,24 @@
 */
 
 /*
- * $Id: ccid_ifdhandler.h,v 1.1.1.3 2005/03/15 18:55:25 rousseau Exp $
+ * $Id: ccid_ifdhandler.h,v 1.2 2005-11-27 20:12:17 rousseau Exp $
  */
 
 #ifndef _ccid_ifd_handler_h_
 #define _ccid_ifd_handler_h_
 
-#define SCARD_CTL_CODE(code) (0x42000000 + (code))
-
 #define IOCTL_SMARTCARD_VENDOR_IFD_EXCHANGE	SCARD_CTL_CODE(1)
-#define IOCTL_SMARTCARD_VENDOR_VERIFY_PIN	SCARD_CTL_CODE(2)
-#define IOCTL_SMARTCARD_VENDOR_MODIFY_PIN	SCARD_CTL_CODE(3)
-#define IOCTL_SMARTCARD_VENDOR_TRANSFER_PIN	SCARD_CTL_CODE(4)
+
+#define CLASS2_IOCTL_MAGIC 0x330000
+#define IOCTL_FEATURE_VERIFY_PIN_DIRECT \
+	SCARD_CTL_CODE(FEATURE_VERIFY_PIN_DIRECT + CLASS2_IOCTL_MAGIC)
+#define IOCTL_FEATURE_MODIFY_PIN_DIRECT \
+	SCARD_CTL_CODE(FEATURE_MODIFY_PIN_DIRECT + CLASS2_IOCTL_MAGIC)
 
 #define DRIVER_OPTION_CCID_EXCHANGE_AUTHORIZED 1
 #define DRIVER_OPTION_GEMPC_TWIN_KEY_APDU 2
 #define DRIVER_OPTION_USE_BOGUS_FIRMWARE 4
+#define DRIVER_OPTION_RESET_ON_CLOSE 8
 
 extern int LogLevel;
 extern int DriverOptions;
