@@ -20,7 +20,7 @@
  */
 
 /*
- * $Id: ccid_serial.c,v 1.1.1.6 2005/03/15 18:55:25 rousseau Exp $
+ * $Id: ccid_serial.c,v 1.2 2005-11-27 20:12:17 rousseau Exp $
  */
 
 #include <stdio.h>
@@ -32,6 +32,7 @@
 #include <errno.h>
 #include <sys/time.h>
 #include <sys/types.h>
+#include <sys/ioctl.h>
 #include <PCSC/ifdhandler.h>
 
 #include "defs.h"
@@ -42,10 +43,6 @@
 #include "utils.h"
 #include "commands.h"
 
-/* communication timeout in seconds
- * the value is set at the end of OpenSerialByName() */
-int SerialTimeout;
-
 #define SYNC 0x03
 #define CTRL_ACK 0x06
 #define CTRL_NAK 0x15
@@ -135,6 +132,50 @@ typedef struct
 /* The _serialDevice structure must be defined before including ccid_serial.h */
 #include "ccid_serial.h"
 
+unsigned int SerialDataRates[] = {
+		10753,
+		14337,
+		15625,
+		17204,
+		20833,
+		21505,
+		23438,
+		25806,
+		28674,
+		31250,
+		32258,
+		34409,
+		39063,
+		41667,
+		43011,
+		46875,
+		52083,
+		53763,
+		57348,
+		62500,
+		64516,
+		68817,
+		71685,
+		78125,
+		83333,
+		86022,
+		93750,
+		104167,
+		107527,
+		114695,
+		125000,
+		129032,
+		143369,
+		156250,
+		166667,
+		172043,
+		215054,
+		229391,
+		250000,
+		344086,
+		0
+	};
+
 /* no need to initialize to 0 since it is static */
 static _serialDevice serialDevice[CCID_DRIVER_MAX_READERS];
 
@@ -158,11 +199,9 @@ status_t WriteSerial(unsigned int reader_index, unsigned int length,
 	unsigned char lrc;
 	unsigned char low_level_buffer[GEMPCTWIN_MAXBUF];
 
-#ifdef DEBUG_LEVEL_COMM
 	char debug_header[] = "-> 123456 ";
 
 	sprintf(debug_header, "-> %06X ", reader_index);
-#endif
 
 	if (length > GEMPCTWIN_MAXBUF-3)
 	{
@@ -184,9 +223,7 @@ status_t WriteSerial(unsigned int reader_index, unsigned int length,
 		lrc ^= low_level_buffer[i];
 	low_level_buffer[length+2] = lrc;
 
-#ifdef DEBUG_LEVEL_COMM
 	DEBUG_XXD(debug_header, low_level_buffer, length+3);
-#endif
 
 	if (write(serialDevice[reader_index].fd, low_level_buffer,
 		length+3) != length+3)
@@ -281,7 +318,10 @@ nak:
 		return STATUS_COMM_ERROR;
 	}
 	else
-		goto start;
+	{
+		DEBUG_COMM("NAK requested");
+		return STATUS_COMM_NAK;
+	}
 
 ack:
 	DEBUG_COMM("ack");
@@ -296,9 +336,7 @@ ack:
 	if ((rv = get_bytes(reader_index, buffer+5, to_read-5)) != STATUS_SUCCESS)
 		return rv;
 
-#ifdef DEBUG_LEVEL_COMM
-		DEBUG_XXD("frame: ", buffer, to_read);
-#endif
+	DEBUG_XXD("frame: ", buffer, to_read);
 
 	/* lrc */
 	DEBUG_COMM("lrc");
@@ -391,11 +429,9 @@ static int ReadChunk(unsigned int reader_index, unsigned char *buffer,
 	struct timeval t;
 	int i, rv = 0;
 	int already_read;
-#ifdef DEBUG_LEVEL_COMM
 	char debug_header[] = "<- 123456 ";
 
 	sprintf(debug_header, "<- %06X ", reader_index);
-#endif
 
 	already_read = 0;
 	while (already_read < min_length)
@@ -403,7 +439,7 @@ static int ReadChunk(unsigned int reader_index, unsigned char *buffer,
 		/* use select() to, eventually, timeout */
 		FD_ZERO(&fdset);
 		FD_SET(fd, &fdset);
-		t.tv_sec = SerialTimeout;
+		t.tv_sec = serialDevice[reader_index].ccid.readTimeout;
 		t.tv_usec = 0;
 
 		i = select(fd+1, &fdset, NULL, NULL, &t);
@@ -415,7 +451,7 @@ static int ReadChunk(unsigned int reader_index, unsigned char *buffer,
 		else
 			if (i == 0)
 			{
-				DEBUG_COMM2("Timeout! (%d sec)", SerialTimeout);
+				DEBUG_COMM2("Timeout! (%d sec)", serialDevice[reader_index].ccid.readTimeout);
 				return -1;
 			}
 
@@ -426,9 +462,7 @@ static int ReadChunk(unsigned int reader_index, unsigned char *buffer,
 			return -1;
 		}
 
-#ifdef DEBUG_LEVEL_COMM
 		DEBUG_XXD(debug_header, buffer + already_read, rv);
-#endif
 
 		already_read += rv;
 		DEBUG_COMM3("read: %d, to read: %d", already_read,
@@ -508,6 +542,30 @@ status_t OpenSerialByName(unsigned int reader_index, char *dev_name)
 		return STATUS_UNSUCCESSFUL;
 	}
 
+	/* Set RTS signal to low to prevent the smart card reader
+	 * from sending its plug and play string. */
+	{
+		int flags;
+
+		if (ioctl(serialDevice[reader].fd, TIOCMGET, &flags) < 0)
+		{
+			DEBUG_CRITICAL2("Get RS232 signals state failed: %s",
+				strerror(errno));
+		}
+		else
+		{
+			flags &= ~TIOCM_RTS;
+			if (ioctl(serialDevice[reader].fd, TIOCMSET, &flags) < 0)
+			{
+				DEBUG_CRITICAL2("Set RTS to low failed: %s", strerror(errno));
+			}
+			else
+			{
+				DEBUG_COMM("Plug-n-Play inhibition successful");
+			}
+		}
+	}
+
 	/* set channel used */
 	serialDevice[reader].device = strdup(dev_name);
 
@@ -563,6 +621,7 @@ status_t OpenSerialByName(unsigned int reader_index, char *dev_name)
 	serialDevice[reader].ccid.dwMaxDataRate = 344086;
 	serialDevice[reader].ccid.bMaxSlotIndex = 0;
 	serialDevice[reader].ccid.bCurrentSlotIndex = 0;
+	serialDevice[reader].ccid.arrayOfSupportedDataRates = SerialDataRates;
 
 	serialDevice[reader].buffer_offset = 0;
 	serialDevice[reader].buffer_offset_last = 0;
@@ -575,7 +634,7 @@ status_t OpenSerialByName(unsigned int reader_index, char *dev_name)
 		unsigned int rx_length = sizeof(rx_buffer);
 
 		/* 2 seconds timeout to not wait too long if no reader is connected */
-		SerialTimeout = 2;
+		serialDevice[reader].ccid.readTimeout = 2;
 
 		if (IFD_SUCCESS != CmdEscape(reader_index, tx_buffer, sizeof(tx_buffer),
 			rx_buffer, &rx_length))
@@ -584,13 +643,29 @@ status_t OpenSerialByName(unsigned int reader_index, char *dev_name)
 			return STATUS_UNSUCCESSFUL;
 		}
 
-		/* normal timeout: 1 minute to allow long time APDU */
-		SerialTimeout = 60;
+		/* normal timeout: 2 seconds */
+		serialDevice[reader].ccid.readTimeout = DEFAULT_COM_READ_TIMEOUT ;
 
 		rx_buffer[rx_length] = '\0';
 		DEBUG_INFO2("Firmware: %s", rx_buffer);
 	}
 
+	/* perform a command to configure GemPC Twin reader card movement
+	 * notification to synchronous mode: the card movement is notified _after_
+	 * the host command and _before_ the reader anwser */
+	{
+		unsigned char tx_buffer[] = { 0x01, 0x01, 0x01};
+		unsigned char rx_buffer[50];
+		unsigned int rx_length = sizeof(rx_buffer);
+
+		if (IFD_SUCCESS != CmdEscape(reader_index, tx_buffer, sizeof(tx_buffer),
+			rx_buffer, &rx_length))
+		{
+			DEBUG_CRITICAL("Change card movement notification failed.");
+			return STATUS_UNSUCCESSFUL;
+		}
+	}
+
 	return STATUS_SUCCESS;
 } /* OpenSerialByName */
 
