@@ -18,7 +18,7 @@
 */
 
 /*
- * $Id: ccid_usb.c,v 1.47 2005/03/14 18:56:51 rousseau Exp $
+ * $Id: ccid_usb.c 1703 2005-11-07 15:46:03Z rousseau $
  */
 
 #define __CCID_USB__
@@ -40,12 +40,6 @@
 #include "ccid_ifdhandler.h"
 
 
-/* read timeout
- * we must wait enough so that the card can finish its calculation
- * the card, and then the reader should send TIME REQUEST bytes
- * so this timeout should never occur */
-#define USB_READ_TIMEOUT (60 * 1000)	/* 1 minute timeout */
-
 /* write timeout
  * we don't have to wait a long time since the card was doing nothing */
 #define USB_WRITE_TIMEOUT (5 * 1000)  /* 5 seconds timeout */
@@ -91,6 +85,7 @@ typedef struct
 
 static int get_end_points(struct usb_device *dev, _usbDevice *usb_device);
 int ccid_check_firmware(struct usb_device *dev);
+static unsigned int *get_data_rates(unsigned int reader_index);
 
 /* ne need to initialize to 0 since it is static */
 static _usbDevice usbDevice[CCID_DRIVER_MAX_READERS];
@@ -140,7 +135,7 @@ status_t OpenUSBByName(unsigned int reader_index, /*@null@*/ char *device)
 	struct usb_bus *bus;
 	struct usb_dev_handle *dev_handle;
 	char keyValue[TOKEN_MAX_VALUE_SIZE];
-	int vendorID, productID;
+	unsigned int vendorID, productID;
 	char infofile[FILENAME_MAX];
 	unsigned int device_vendor, device_product;
 	char *dirname = NULL, *filename = NULL;
@@ -344,6 +339,7 @@ status_t OpenUSBByName(unsigned int reader_index, /*@null@*/ char *device)
 					/* now we found a free reader and we try to use it */
 					if (dev->config == NULL)
 					{
+						usb_close(dev_handle);
 						DEBUG_CRITICAL3("No dev->config found for %s/%s",
 							 bus->dirname, dev->filename);
 						return STATUS_UNSUCCESSFUL;
@@ -352,6 +348,7 @@ status_t OpenUSBByName(unsigned int reader_index, /*@null@*/ char *device)
 					usb_interface = get_ccid_usb_interface(dev);
 					if (usb_interface == NULL)
 					{
+						usb_close(dev_handle);
 						DEBUG_CRITICAL3("Can't find a CCID interface on %s/%s",
 							bus->dirname, dev->filename);
 						return STATUS_UNSUCCESSFUL;
@@ -359,6 +356,7 @@ status_t OpenUSBByName(unsigned int reader_index, /*@null@*/ char *device)
 
 					if (usb_interface->altsetting->extralen != 54)
 					{
+						usb_close(dev_handle);
 						DEBUG_CRITICAL4("Extra field for %s/%s has a wrong length: %d", bus->dirname, dev->filename, usb_interface->altsetting->extralen);
 						return STATUS_UNSUCCESSFUL;
 					}
@@ -366,6 +364,7 @@ status_t OpenUSBByName(unsigned int reader_index, /*@null@*/ char *device)
 					interface = usb_interface->altsetting->bInterfaceNumber;
 					if (usb_claim_interface(dev_handle, interface) < 0)
 					{
+						usb_close(dev_handle);
 						DEBUG_CRITICAL4("Can't claim interface %s/%s: %s",
 							bus->dirname, dev->filename, strerror(errno));
 						return STATUS_UNSUCCESSFUL;
@@ -379,7 +378,10 @@ status_t OpenUSBByName(unsigned int reader_index, /*@null@*/ char *device)
 
 					/* check for firmware bugs */
 					if (ccid_check_firmware(dev))
+					{
+						usb_close(dev_handle);
 						return STATUS_UNSUCCESSFUL;
+					}
 
 					/* Get Endpoints values*/
 					get_end_points(dev, &usbDevice[reader_index]);
@@ -403,6 +405,8 @@ status_t OpenUSBByName(unsigned int reader_index, /*@null@*/ char *device)
 					usbDevice[reader_index].ccid.dwMaxDataRate = dw2i(usb_interface->altsetting->extra, 23);
 					usbDevice[reader_index].ccid.bMaxSlotIndex = usb_interface->altsetting->extra[4];
 					usbDevice[reader_index].ccid.bCurrentSlotIndex = 0;
+					usbDevice[reader_index].ccid.readTimeout = DEFAULT_COM_READ_TIMEOUT;
+					usbDevice[reader_index].ccid.arrayOfSupportedDataRates = get_data_rates(reader_index);
 					goto end;
 				}
 			}
@@ -429,15 +433,11 @@ status_t WriteUSB(unsigned int reader_index, unsigned int length,
 	unsigned char *buffer)
 {
 	int rv;
-#ifdef DEBUG_LEVEL_COMM
 	char debug_header[] = "-> 121234 ";
 
 	sprintf(debug_header, "-> %06X ", (int)reader_index);
-#endif
 
-#ifdef DEBUG_LEVEL_COMM
 	DEBUG_XXD(debug_header, buffer, length);
-#endif
 
 	rv = usb_bulk_write(usbDevice[reader_index].handle,
 		usbDevice[reader_index].bulk_out, (char *)buffer, length,
@@ -445,9 +445,15 @@ status_t WriteUSB(unsigned int reader_index, unsigned int length,
 
 	if (rv < 0)
 	{
-		DEBUG_CRITICAL4("usb_bulk_write(%s/%s): %s",
-			usbDevice[reader_index].dev->bus->dirname,
-			usbDevice[reader_index].dev->filename, strerror(errno));
+		if (usbDevice[reader_index].dev->bus)
+		{
+			DEBUG_CRITICAL4("usb_bulk_write(%s/%s): %s",
+				usbDevice[reader_index].dev->bus->dirname,
+				usbDevice[reader_index].dev->filename, strerror(errno));
+		}
+		else
+			DEBUG_CRITICAL2("usb_bulk_write(no device): %s", strerror(errno));
+
 		return STATUS_UNSUCCESSFUL;
 	}
 
@@ -464,15 +470,13 @@ status_t ReadUSB(unsigned int reader_index, unsigned int * length,
 	unsigned char *buffer)
 {
 	int rv;
-#ifdef DEBUG_LEVEL_COMM
 	char debug_header[] = "<- 121234 ";
 
 	sprintf(debug_header, "<- %06X ", (int)reader_index);
-#endif
 
 	rv = usb_bulk_read(usbDevice[reader_index].handle,
 		usbDevice[reader_index].bulk_in, (char *)buffer, *length,
-		USB_READ_TIMEOUT);
+		usbDevice[reader_index].ccid.readTimeout * 1000);
 
 	if (rv < 0)
 	{
@@ -485,9 +489,7 @@ status_t ReadUSB(unsigned int reader_index, unsigned int * length,
 
 	*length = rv;
 
-#ifdef DEBUG_LEVEL_COMM
 	DEBUG_XXD(debug_header, buffer, *length);
-#endif
 
 	return STATUS_SUCCESS;
 } /* ReadUSB */
@@ -508,8 +510,12 @@ status_t CloseUSB(unsigned int reader_index)
 		usbDevice[reader_index].dev->bus->dirname,
 		usbDevice[reader_index].dev->filename);
 
+	if (usbDevice[reader_index].ccid.arrayOfSupportedDataRates)
+		free(usbDevice[reader_index].ccid.arrayOfSupportedDataRates);
+
 	/* reset so that bSeq starts at 0 again */
-	usb_reset(usbDevice[reader_index].handle);
+	if (DriverOptions & DRIVER_OPTION_RESET_ON_CLOSE)
+		usb_reset(usbDevice[reader_index].handle);
 
 	usb_release_interface(usbDevice[reader_index].handle,
 		usbDevice[reader_index].interface);
@@ -686,3 +692,74 @@ int ccid_check_firmware(struct usb_device *dev)
 	return FALSE;
 } /* ccid_check_firmware */
 
+/*****************************************************************************
+ *
+ *                                      get_data_rates
+ *
+ ****************************************************************************/
+static unsigned int *get_data_rates(unsigned int reader_index)
+{
+	int n, i, len;
+	unsigned char buffer[256*sizeof(int)];	/* maximum is 256 records */
+	unsigned int *int_array;
+
+	/* See CCID 3.7.3 page 25 */
+	n = usb_control_msg(usbDevice[reader_index].handle,
+		0xA1, /* request type */
+		0x03, /* GET_DATA_RATES */
+		0x00, /* value */
+		usbDevice[reader_index].interface, /* interface */
+		(char *)buffer,
+		sizeof(buffer),
+		usbDevice[reader_index].ccid.readTimeout * 1000);
+
+	/* we got an error? */
+	if (n <= 0)
+	{
+		DEBUG_INFO2("IFD does not support GET_DATA_RATES request: %s",
+			strerror(errno));
+		return NULL;
+	}
+
+	/* we got a strange value */
+	if (n % 4)
+	{
+		DEBUG_INFO2("Wrong GET DATA RATES size: %d", n);
+		return NULL;
+	}
+
+	/* allocate the buffer (including the end marker) */
+	n /= sizeof(int);
+
+	/* we do not get the expected number of data rates */
+	len = get_ccid_usb_interface(usbDevice[reader_index].dev)
+		->altsetting->extra[27]; /* bNumDataRatesSupported */
+	if ((n != len) && len)
+	{
+		DEBUG_INFO3("Got %d data rates but was expecting %d", n, len);
+
+		/* we got more data than expected */
+		if (n > len)
+			n = len;
+	}
+
+	int_array = calloc(n+1, sizeof(int));
+	if (NULL == int_array)
+	{
+		DEBUG_CRITICAL("Memory allocation failed");
+		return NULL;
+	}
+
+	/* convert in correct endianess */
+	for (i=0; i<n; i++)
+	{
+		int_array[i] = dw2i(buffer, i*4);
+		DEBUG_INFO2("declared: %d bps", int_array[i]);
+	}
+
+	/* end of array marker */
+	int_array[i] = 0;
+
+	return int_array;
+}
+
