@@ -18,7 +18,7 @@
 */
 
 /*
- * $Id: commands.c,v 1.38 2005/03/14 18:56:51 rousseau Exp $
+ * $Id: commands.c 1747 2005-11-24 15:49:00Z rousseau $
  */
 
 #include <string.h>
@@ -47,6 +47,8 @@
  */
 #define DEFAULT_VOLTAGE 1 /* start with 5 volts */
 
+#define max( a, b )   ( ( ( a ) > ( b ) ) ? ( a ) : ( b ) )
+
 /* internal functions */
 static RESPONSECODE CmdXfrBlockTPDU_T0(unsigned int reader_index,
 	unsigned int tx_length, unsigned char tx_buffer[], unsigned int *rx_length,
@@ -73,7 +75,8 @@ RESPONSECODE CmdPowerOn(unsigned int reader_index, unsigned int * nlength,
 {
 	unsigned char cmd[10];
 	status_t res;
-	int atr_len, length, count = 1;
+	int length, count = 1;
+	unsigned int atr_len;
 	RESPONSECODE return_value = IFD_SUCCESS;
 	_ccid_descriptor *ccid_descriptor = get_ccid_descriptor(reader_index);
 	char voltage;
@@ -154,18 +157,20 @@ again:
 
 /*****************************************************************************
  *
- *					SecurePIN
+ *					SecurePINVerify
  *
  ****************************************************************************/
-RESPONSECODE SecurePIN(unsigned int reader_index,
+RESPONSECODE SecurePINVerify(unsigned int reader_index,
 	const unsigned char TxBuffer[], unsigned int TxLength,
 	unsigned char RxBuffer[], unsigned int *RxLength)
 {
 	unsigned char cmd[11+14+CMD_BUF_SIZE];
+	unsigned int a, b;
 	_ccid_descriptor *ccid_descriptor = get_ccid_descriptor(reader_index);
+	int old_read_timeout;
+	RESPONSECODE ret;
 
 	cmd[0] = 0x69;	/* Secure */
-	i2dw(TxLength+1, cmd+1);	/* command length (includes bPINOperation) */
 	cmd[5] = ccid_descriptor->bCurrentSlotIndex;	/* slot number */
 	cmd[6] = (*ccid_descriptor->pbSeq)++;
 	cmd[7] = 0;		/* bBWI */
@@ -173,18 +178,52 @@ RESPONSECODE SecurePIN(unsigned int reader_index,
 	cmd[9] = 0;
 	cmd[10] = 0;	/* bPINOperation: PIN Verification */
 
-	/* check that the command is not too large */
-	if (TxLength > 14+CMD_BUF_SIZE)
+	/* 19 is the size of the PCSCv2 PIN verify structure
+	 * The equivalent CCID structure is only 14-bytes long */
+	if (TxLength > 19+CMD_BUF_SIZE) /* command too large? */
+	{
+		DEBUG_INFO3("Command too long: %d > %d", TxLength, 16+CMD_BUF_SIZE);
+		*RxLength = 0;
 		return IFD_NOT_SUPPORTED;
+	}
 
-	/* CCID data structure + APDU */
-	memcpy(cmd + 11, TxBuffer, TxLength);
+	if (TxLength < 19+4 /* 4 = APDU size */)	/* command too short? */
+	{
+		DEBUG_INFO3("Command too short: %d < %d", TxLength, 19+4);
+		*RxLength = 0;
+		return IFD_NOT_SUPPORTED;
+	}
+
+	if (dw2i(TxBuffer, 15) + 19 != TxLength) /* ulDataLength field coherency */
+	{
+		DEBUG_INFO3("Wrong lengths: %d %d", TxBuffer[15] + 19, TxLength);
+		*RxLength = 0;
+		return IFD_NOT_SUPPORTED;
+	}
+
+	/* Build a CCID block from a PC/SC V2.1.2 Part 10 block */
+	for (a = 11, b = 0; b < TxLength; b++)
+	{
+		if (1 == b) /* bTimeOut2 field */
+			/* Ignore the second timeout as there's nothing we can do with
+			 * it currently */
+			continue;
+
+		if ((b >= 15) && (b <= 18)) /* ulDataLength field (4 bytes) */
+			/* the ulDataLength field is not present in the CCID frame
+			 * so do not copy */
+			continue;
+
+		/* copy the CCID block 'verbatim' */
+		cmd[a] = TxBuffer[b];
+		a++;
+	}
 
 	/* SPR532 and Case 1 APDU */
-	if ((SPR532 == ccid_descriptor->readerID) && (TxLength - 14 == 4))
+	if ((SPR532 == ccid_descriptor->readerID) && (TxBuffer[15] == 4))
 	{
 		RESPONSECODE return_value;
-		unsigned char cmd[] = { 0x80,0x02, 0x00 };
+		unsigned char cmd[] = { 0x80, 0x02, 0x00 };
 		unsigned char res[1];
 		unsigned int res_length = sizeof(res);
 
@@ -198,11 +237,131 @@ RESPONSECODE SecurePIN(unsigned int reader_index,
 		}
 	}
 
-	if (WritePort(reader_index, TxLength+1+10, cmd) != STATUS_SUCCESS)
-		return IFD_COMMUNICATION_ERROR;
+	i2dw(a - 10, cmd + 1);  /* CCID message length */
 
-	return CCID_Receive(reader_index, RxLength, RxBuffer);
-} /* SecurePIN */
+	old_read_timeout = ccid_descriptor -> readTimeout;
+	ccid_descriptor -> readTimeout = max(30, TxBuffer[0]);	/* at least 30 seconds */
+
+	if (WritePort(reader_index, a, cmd) != STATUS_SUCCESS)
+	{
+		*RxLength = 0;
+		return IFD_COMMUNICATION_ERROR;
+	}
+
+	ret = CCID_Receive(reader_index, RxLength, RxBuffer);
+
+	ccid_descriptor -> readTimeout = old_read_timeout;
+	return ret;
+} /* SecurePINVerify */
+
+
+/*****************************************************************************
+ *
+ *					SecurePINModify
+ *
+ ****************************************************************************/
+RESPONSECODE SecurePINModify(unsigned int reader_index,
+	const unsigned char TxBuffer[], unsigned int TxLength,
+	unsigned char RxBuffer[], unsigned int *RxLength)
+{
+	unsigned char cmd[11+19+CMD_BUF_SIZE];
+	unsigned int a, b;
+	_ccid_descriptor *ccid_descriptor = get_ccid_descriptor(reader_index);
+	int old_read_timeout;
+	RESPONSECODE ret;
+
+	cmd[0] = 0x69;	/* Secure */
+	cmd[5] = ccid_descriptor->bCurrentSlotIndex;	/* slot number */
+	cmd[6] = (*ccid_descriptor->pbSeq)++;
+	cmd[7] = 0;		/* bBWI */
+	cmd[8] = 0;		/* wLevelParameter */
+	cmd[9] = 0;
+	cmd[10] = 1;	/* bPINOperation: PIN Modification */
+
+	/* 24 is the size of the PCSC PIN modify structure
+	 * The equivalent CCID structure is only 18 or 19-bytes long */
+	if ((TxLength > 19+CMD_BUF_SIZE) /* command too large? */
+		|| (TxLength < 18+4 /* 4 = APDU size */) /* command too short? */
+		|| (TxBuffer[20] + 24 != TxLength)) /* ulDataLength field coherency */
+	{
+		*RxLength = 0;
+		return IFD_NOT_SUPPORTED;
+	}
+
+	/* Make sure in the beginning if bNumberMessage is valid or not */
+	if (TxBuffer[11] > 3)
+	{
+		*RxLength = 0;
+		return IFD_NOT_SUPPORTED;
+	}
+
+	/* Build a CCID block from a PC/SC V2.1.2 Part 10 block */
+
+	/* Do adjustments as needed - CCID spec is not exact with some
+	 * details in the format of the structure, per-reader adaptions
+	 * might be needed.
+	 */
+	for (a = 11, b = 0; b < TxLength; b++)
+	{
+		if (1 == b) /* bTimeOut2 */
+			/* Ignore the second timeout as there's nothing we can do with it
+			 * currently */
+			continue;
+
+		if (15 == b) /* bMsgIndex2 */
+		{
+			/* in CCID the bMsgIndex2 is present only if bNumberMessage != 0 */
+			if (0 == TxBuffer[11])
+				continue;
+		}
+
+		if (16 == b) /* bMsgIndex3 */
+		{
+			/*
+			 * SPR 532 and Cherry ST 2000C has no display but requires _all_
+			 * bMsgIndex fields with bNumberMessage set to 0.
+			 */
+			if ((SPR532 == ccid_descriptor->readerID)
+				|| (CHERRYST2000 == ccid_descriptor->readerID))
+			{
+				cmd[21] = 0x00; /* set bNumberMessages to 0 */
+				cmd[a] = cmd[a-1] = cmd[a+1] = 0;	/* bMsgIndex123 */
+				a += 2;
+				continue;
+			}
+
+			/* in CCID the bMsgIndex3 is present only if bNumberMessage == 3 */
+			if (TxBuffer[11] < 3)
+				continue;
+		}
+
+		if ((b >= 20) && (b <= 23)) /* ulDataLength field (4 bytes) */
+			/* the ulDataLength field is not present in the CCID frame
+			 * so do not copy */
+			continue;
+
+		/* copy to the CCID block 'verbatim' */
+		cmd[a] = TxBuffer[b];
+		a++;
+ 	}
+
+	/* We know the size of the CCID message now */
+	i2dw(a - 10, cmd + 1);	/* command length (includes bPINOperation) */
+
+	old_read_timeout = ccid_descriptor -> readTimeout;
+	ccid_descriptor -> readTimeout = max(30, TxBuffer[0]);	/* at least 30 seconds */
+
+	if (WritePort(reader_index, a, cmd) != STATUS_SUCCESS)
+	{
+		*RxLength = 0;
+ 		return IFD_COMMUNICATION_ERROR;
+	}
+
+ 	ret = CCID_Receive(reader_index, RxLength, RxBuffer);
+
+	ccid_descriptor -> readTimeout = old_read_timeout;
+	return ret;
+} /* SecurePINModify */
 
 
 /*****************************************************************************
@@ -220,6 +379,7 @@ RESPONSECODE CmdEscape(unsigned int reader_index,
 	RESPONSECODE return_value = IFD_SUCCESS;
 	_ccid_descriptor *ccid_descriptor = get_ccid_descriptor(reader_index);
 
+again:
 	/* allocate buffers */
 	length_in = 10 + TxLength;
 	if (NULL == (cmd_in = malloc(length_in)))
@@ -250,6 +410,17 @@ RESPONSECODE CmdEscape(unsigned int reader_index,
 	}
 
 	res = ReadPort(reader_index, &length_out, cmd_out);
+
+	/* replay the command if NAK
+	 * This (generally) happens only for the first command sent to the reader
+	 * with the serial protocol so it is not really needed for all the other
+	 * ReadPort() calls */
+	if (STATUS_COMM_NAK == res)
+	{
+		free(cmd_out);
+		goto again;
+	}
+
 	if (res != STATUS_SUCCESS)
 	{
 		free(cmd_out);
@@ -445,7 +616,7 @@ RESPONSECODE CCID_Transmit(unsigned int reader_index, unsigned int tx_length,
 	cmd[5] = ccid_descriptor->bCurrentSlotIndex;	/* slot number */
 	cmd[6] = (*ccid_descriptor->pbSeq)++;
 	cmd[7] = bBWI;	/* extend block waiting timeout */
-	cmd[8] = rx_length & 0xFF;	/* Expected length */
+	cmd[8] = rx_length & 0xFF;	/* Expected length, in character mode only */
 	cmd[9] = (rx_length >> 8) & 0xFF;
 
 	/* check that the command is not too large */
@@ -483,16 +654,37 @@ time_request:
 	if (cmd[STATUS_OFFSET] & CCID_COMMAND_FAILED)
 	{
 		ccid_error(cmd[ERROR_OFFSET], __FILE__, __LINE__, __FUNCTION__);    /* bError */
-		*rx_length = 0; /* nothing received */
-		if (0xFD == cmd[ERROR_OFFSET]) /* Parity error during exchange */
-			return IFD_PARITY_ERROR;
-		else
-			return IFD_COMMUNICATION_ERROR;
+		switch (cmd[ERROR_OFFSET])
+		{
+			case 0xEF:	/* cancel */
+				if (*rx_length < 2)
+					return IFD_COMMUNICATION_ERROR;
+				rx_buffer[0]= 0x64;
+				rx_buffer[1]= 0x01;
+				*rx_length = 2;
+				return IFD_SUCCESS;
+
+			case 0xF0:	/* timeout */
+				if (*rx_length < 2)
+					return IFD_COMMUNICATION_ERROR;
+				rx_buffer[0]= 0x64;
+				rx_buffer[1]= 0x00;
+				*rx_length = 2;
+				return IFD_SUCCESS;
+
+			case 0xFD:	/* Parity error during exchange */
+				*rx_length = 0; /* nothing received */
+				return IFD_PARITY_ERROR;
+
+			default:
+				*rx_length = 0; /* nothing received */
+				return IFD_COMMUNICATION_ERROR;
+		}
 	}
 
 	if (cmd[STATUS_OFFSET] & CCID_TIME_EXTENSION)
 	{
-		DEBUG_CRITICAL2("Time extension requested: 0x%02X", cmd[ERROR_OFFSET]);
+		DEBUG_COMM2("Time extension requested: 0x%02X", cmd[ERROR_OFFSET]);
 		goto time_request;
 	}
 
