@@ -18,7 +18,7 @@
 */
 
 /*
- * $Id: commands.h,v 1.14 2005/03/14 18:56:52 rousseau Exp $
+ * $Id: commands.h 1630 2005-09-01 09:11:30Z rousseau $
  */
 
 #define SIZE_GET_SLOT_STATUS 10
@@ -28,7 +28,11 @@
 RESPONSECODE CmdPowerOn(unsigned int reader_index, unsigned int * nlength,
 	unsigned char buffer[]);
 
-RESPONSECODE SecurePIN(unsigned int reader_index,
+RESPONSECODE SecurePINVerify(unsigned int reader_index,
+	const unsigned char TxBuffer[], unsigned int TxLength,
+	unsigned char RxBuffer[], unsigned int *RxLength);
+
+RESPONSECODE SecurePINModify(unsigned int reader_index,
 	const unsigned char TxBuffer[], unsigned int TxLength,
 	unsigned char RxBuffer[], unsigned int *RxLength);
 
