@@ -1,6 +1,6 @@
 /*
     ifdhandler.c: IFDH API
-    Copyright (C) 2003   Ludovic Rousseau
+    Copyright (C) 2003-2005   Ludovic Rousseau
 
     This library is free software; you can redistribute it and/or
     modify it under the terms of the GNU Lesser General Public
@@ -17,13 +17,14 @@
     Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA
 */
 
-/* $Id: ifdhandler.c,v 1.59 2005/03/14 18:56:52 rousseau Exp $ */
+/* $Id: ifdhandler.c 1752 2005-11-24 16:16:33Z rousseau $ */
 
 #include <stdio.h>
 #include <string.h>
 #include <stdlib.h>
 #include <PCSC/pcsclite.h>
 #include <PCSC/ifdhandler.h>
+#include <PCSC/reader.h>
 
 #include "ccid.h"
 #include "defs.h"
@@ -54,6 +55,12 @@ static int DebugInitialized = FALSE;
 
 /* local functions */
 static void init_driver(void);
+static void extra_egt(ATR_t *atr, _ccid_descriptor *ccid_desc, DWORD Protocol);
+static char find_baud_rate(unsigned int baudrate, unsigned int *list);
+static unsigned int T0_card_timeout(double f, double d, int TC1, int TC2, 
+	int clock_frequency);
+static unsigned int T1_card_timeout(double f, double d, int TC1, int BWI,
+	int CWI, int clock_frequency);
 
 
 RESPONSECODE IFDHCreateChannelByName(DWORD Lun, LPTSTR lpcDevice)
@@ -196,6 +203,10 @@ RESPONSECODE IFDHCloseChannel(DWORD Lun)
 	if (-1 == (reader_index = LunToReaderIndex(Lun)))
 		return IFD_COMMUNICATION_ERROR;
 
+	/* Restore the default timeout
+	 * No need to wait too long if the reader disapeared */
+	get_ccid_descriptor(reader_index)->readTimeout = DEFAULT_COM_READ_TIMEOUT;
+
 	(void)CmdPowerOff(reader_index);
 	/* No reader status check, if it failed, what can you do ? :) */
 
@@ -252,8 +263,7 @@ RESPONSECODE IFDHGetCapabilities(DWORD Lun, DWORD Tag,
 				*Length : CcidSlots[reader_index].nATRLength;
 
 			if (*Length)
-				memcpy(Value, CcidSlots[reader_index]
-					.pcATRBuffer, *Length);
+				memcpy(Value, CcidSlots[reader_index].pcATRBuffer, *Length);
 			break;
 
 #ifdef HAVE_PTHREAD
@@ -279,7 +289,7 @@ RESPONSECODE IFDHGetCapabilities(DWORD Lun, DWORD Tag,
 			{
 				*Length = 1;
 				*Value = 1 + get_ccid_descriptor(reader_index) -> bMaxSlotIndex;
-				DEBUG_COMM2("Reader supports %d slots", *Value);
+				DEBUG_INFO2("Reader supports %d slots", *Value);
 			}
 			break;
 
@@ -291,14 +301,6 @@ RESPONSECODE IFDHGetCapabilities(DWORD Lun, DWORD Tag,
 			}
 			break;
 
-		case IOCTL_SMARTCARD_VENDOR_VERIFY_PIN:
-			if (*Length >= 1)
-			{
-				*Length = 1;
-				*Value = get_ccid_descriptor(reader_index) -> bPINSupport & CCID_CLASS_PIN_VERIFY;
-			}
-			break;
-
 		default:
 			return IFD_ERROR_TAG;
 	}
@@ -391,6 +393,9 @@ RESPONSECODE IFDHSetProtocolParameters(DWORD Lun, DWORD Protocol,
 	/* Get ATR of the card */
 	ATR_InitFromArray(&atr, ccid_slot->pcATRBuffer, ccid_slot->nATRLength);
 
+	/* Apply Extra EGT patch for bogus cards */
+	extra_egt(&atr, ccid_desc, Protocol);
+
 	if (SCARD_PROTOCOL_T0 == Protocol)
 		pps[1] |= ATR_PROTOCOL_TYPE_T0;
 	else
@@ -468,15 +473,25 @@ RESPONSECODE IFDHSetProtocolParameters(DWORD Lun, DWORD Protocol,
 			default_baudrate = (unsigned int) (1000 * ccid_desc->dwDefaultClock
 				* ATR_DEFAULT_D / ATR_DEFAULT_F);
 
-			/* if the reader is fast enough */
-			if ((card_baudrate <= ccid_desc->dwMaxDataRate)
-				/* and the card does not try to lower the default speed */
-				&& (card_baudrate > default_baudrate ))
+			/* if the card does not try to lower the default speed */
+			if ((card_baudrate > default_baudrate)
+				/* and the reader is fast enough */
+				&& (card_baudrate <= ccid_desc->dwMaxDataRate))
 			{
-				pps[1] |= 0x10; /* PTS1 presence */
-				pps[2] = atr.ib[0][ATR_INTERFACE_BYTE_TA].value;
+				/* the reader has no baud rates table */
+				if ((NULL == ccid_desc->arrayOfSupportedDataRates)
+					/* or explicitely support it */
+					|| find_baud_rate(card_baudrate,
+						ccid_desc->arrayOfSupportedDataRates))
+				{
+					pps[1] |= 0x10; /* PTS1 presence */
+					pps[2] = atr.ib[0][ATR_INTERFACE_BYTE_TA].value;
 
-				DEBUG_COMM2("Set speed to %d bauds", card_baudrate);
+					DEBUG_COMM2("Set speed to %d bauds", card_baudrate);
+				}
+				else
+					DEBUG_COMM2("Reader does not support %d bauds",
+						card_baudrate);
 			}
 		}
 	}
@@ -534,7 +549,7 @@ RESPONSECODE IFDHSetProtocolParameters(DWORD Lun, DWORD Protocol,
 			0x11,	/* Fi/Di		*/
 			0x10,	/* TCCKS		*/
 			0x00,	/* GuardTime	*/
-			0x4D,	/* BWI/BCI		*/
+			0x4D,	/* BWI/CWI		*/
 			0x00,	/* ClockStop	*/
 			0x20,	/* IFSC			*/
 			0x00	/* NADValue		*/
@@ -542,11 +557,12 @@ RESPONSECODE IFDHSetProtocolParameters(DWORD Lun, DWORD Protocol,
 		int i;
 		t1_state_t *t1 = &(ccid_slot -> t1);
 		RESPONSECODE ret;
+		double f, d;
 
 		/* TA1 is not default */
 		if (PPS_HAS_PPS1(pps))
 			param[0] = pps[2];
- 
+
 		/* CRC checksum? */
 		if (2 == t1->rc_bytes)
 			param[1] |= 0x01;
@@ -559,11 +575,11 @@ RESPONSECODE IFDHSetProtocolParameters(DWORD Lun, DWORD Protocol,
 		if (atr.ib[0][ATR_INTERFACE_BYTE_TC].present)
 			param[2] = atr.ib[0][ATR_INTERFACE_BYTE_TC].value;
 
-		/* TBi (i>2) present? BWI/BCI */
+		/* TBi (i>2) present? BWI/CWI */
 		for (i=2; i<ATR_MAX_PROTOCOLS; i++)
 			if (atr.ib[i][ATR_INTERFACE_BYTE_TB].present)
 			{
-				DEBUG_COMM3("BWI/BCI (TB%d) present: %d", i+1,
+				DEBUG_COMM3("BWI/CWI (TB%d) present: 0x%02X", i+1,
 					atr.ib[i][ATR_INTERFACE_BYTE_TB].value);
 				param[3] = atr.ib[i][ATR_INTERFACE_BYTE_TB].value;
 
@@ -571,6 +587,15 @@ RESPONSECODE IFDHSetProtocolParameters(DWORD Lun, DWORD Protocol,
 				break;
 			}
 
+		/* compute communication timeout */
+		ATR_GetParameter(&atr, ATR_PARAMETER_F, &f);
+		ATR_GetParameter(&atr, ATR_PARAMETER_D, &d);
+		ccid_desc->readTimeout = T1_card_timeout(f, d, param[2],
+			(param[3] & 0xF0) >> 4 /* BWI */, param[3] & 0x0F /* CWI */,
+			ccid_desc->dwDefaultClock);
+
+		DEBUG_COMM2("Timeout: %d seconds", ccid_desc->readTimeout);
+
 		ret = SetParameters(reader_index, 1, sizeof(param), param);
 		if (IFD_SUCCESS != ret)
 			return ret;
@@ -586,6 +611,7 @@ RESPONSECODE IFDHSetProtocolParameters(DWORD Lun, DWORD Protocol,
 			0x00	/* ClockStop		*/
 		};
 		RESPONSECODE ret;
+		double f, d;
 
 		/* TA1 is not default */
 		if (PPS_HAS_PPS1(pps))
@@ -602,6 +628,16 @@ RESPONSECODE IFDHSetProtocolParameters(DWORD Lun, DWORD Protocol,
 		if (atr.ib[1][ATR_INTERFACE_BYTE_TC].present)
 			param[3] = atr.ib[1][ATR_INTERFACE_BYTE_TC].value;
 
+		/* compute communication timeout */
+		ATR_GetParameter(&atr, ATR_PARAMETER_F, &f);
+		ATR_GetParameter(&atr, ATR_PARAMETER_D, &d);
+
+		ccid_desc->readTimeout = T0_card_timeout(f, d, param[2] /* TC1 */,
+			param[3] /* TC2 */, ccid_desc->dwDefaultClock);
+
+		DEBUG_COMM2("Communication timeout: %d seconds",
+			ccid_desc->readTimeout);
+
 		ret = SetParameters(reader_index, 0, sizeof(param), param);
 		if (IFD_SUCCESS != ret)
 			return ret;
@@ -629,7 +665,7 @@ RESPONSECODE IFDHSetProtocolParameters(DWORD Lun, DWORD Protocol,
 		/* IFSD not negociated by the reader? */
 		if (! (ccid_desc->dwFeatures & CCID_CLASS_AUTO_IFSD))
 		{
-			DEBUG_COMM2("Negociate IFSD at %d",  ccid_desc -> dwMaxIFSD);
+			DEBUG_COMM2("Negociate IFSD at %d", ccid_desc -> dwMaxIFSD);
 			if (t1_negociate_ifsd(t1, 0, ccid_desc -> dwMaxIFSD) < 0)
 				return IFD_COMMUNICATION_ERROR;
 		}
@@ -702,8 +738,7 @@ RESPONSECODE IFDHPowerICC(DWORD Lun, DWORD Action,
 			*CcidSlots[reader_index].pcATRBuffer = '\0';
 
 			/* Memorise the request */
-			CcidSlots[reader_index].bPowerFlags |=
-				MASK_POWERFLAGS_PDWN;
+			CcidSlots[reader_index].bPowerFlags |= MASK_POWERFLAGS_PDWN;
 
 			/* send the command */
 			if (IFD_SUCCESS != CmdPowerOff(reader_index))
@@ -728,17 +763,14 @@ RESPONSECODE IFDHPowerICC(DWORD Lun, DWORD Action,
 			}
 
 			/* Power up successful, set state variable to memorise it */
-			CcidSlots[reader_index].bPowerFlags |=
-				MASK_POWERFLAGS_PUP;
-			CcidSlots[reader_index].bPowerFlags &=
-				~MASK_POWERFLAGS_PDWN;
+			CcidSlots[reader_index].bPowerFlags |= MASK_POWERFLAGS_PUP;
+			CcidSlots[reader_index].bPowerFlags &= ~MASK_POWERFLAGS_PDWN;
 
 			/* Reset is returned, even if TCK is wrong */
 			CcidSlots[reader_index].nATRLength = *AtrLength =
 				(nlength < MAX_ATR_SIZE) ? nlength : MAX_ATR_SIZE;
 			memcpy(Atr, pcbuffer, *AtrLength);
-			memcpy(CcidSlots[reader_index].pcATRBuffer,
-				pcbuffer, *AtrLength);
+			memcpy(CcidSlots[reader_index].pcATRBuffer, pcbuffer, *AtrLength);
 
 			/* initialise T=1 context */
 			t1_init(&(get_ccid_slot(reader_index) -> t1), reader_index);
@@ -833,8 +865,7 @@ RESPONSECODE IFDHControl(DWORD Lun, DWORD dwControlCode, PUCHAR TxBuffer,
 	DEBUG_INFO3("lun: %X, ControlCode: 0x%X", Lun, dwControlCode);
 
 	reader_index = LunToReaderIndex(Lun);
-	if ((-1 == reader_index) || (NULL == pdwBytesReturned)
-		|| (NULL == RxBuffer))
+	if ((-1 == reader_index) || (NULL == pdwBytesReturned))
 		return return_value;
 
 	/* Set the return length to 0 to avoid problems */
@@ -858,13 +889,63 @@ RESPONSECODE IFDHControl(DWORD Lun, DWORD dwControlCode, PUCHAR TxBuffer,
 		}
 	}
 
-	if (IOCTL_SMARTCARD_VENDOR_VERIFY_PIN == dwControlCode)
+	/* Implement the PC/SC v2.1.2 Part 10 IOCTL mechanism */
+	
+	/* Query for features */
+	if (CM_IOCTL_GET_FEATURE_REQUEST == dwControlCode)
+	{
+		unsigned int iBytesReturned = 0;
+		PCSC_TLV_STRUCTURE *pcsc_tlv = (PCSC_TLV_STRUCTURE *)RxBuffer;
+
+		/* we need room for two records */
+		if (RxLength < 2 * sizeof(PCSC_TLV_STRUCTURE))
+			return IFD_COMMUNICATION_ERROR;
+
+		/* We can only support direct verify and/or modify currently */
+		if (get_ccid_descriptor(reader_index) -> bPINSupport
+			& CCID_CLASS_PIN_VERIFY)
+		{
+			pcsc_tlv -> tag = FEATURE_VERIFY_PIN_DIRECT;
+			pcsc_tlv -> length = 0x04; /* always 0x04 */
+			pcsc_tlv -> value = IOCTL_FEATURE_VERIFY_PIN_DIRECT;
+
+			pcsc_tlv++;
+			iBytesReturned += sizeof(PCSC_TLV_STRUCTURE);
+		}
+		
+		if (get_ccid_descriptor(reader_index) -> bPINSupport
+			& CCID_CLASS_PIN_MODIFY)
+		{
+			pcsc_tlv -> tag = FEATURE_MODIFY_PIN_DIRECT;
+			pcsc_tlv -> length = 0x04; /* always 0x04 */
+			pcsc_tlv -> value = IOCTL_FEATURE_MODIFY_PIN_DIRECT;
+
+			pcsc_tlv++;
+			iBytesReturned += sizeof(PCSC_TLV_STRUCTURE);
+		}
+		*pdwBytesReturned = iBytesReturned;
+		return_value = IFD_SUCCESS;
+	}
+
+	/* Verify a PIN, plain CCID */
+	if (IOCTL_FEATURE_VERIFY_PIN_DIRECT == dwControlCode)
 	{
 		unsigned int iBytesReturned;
 
 		iBytesReturned = RxLength;
-		return_value = SecurePIN(reader_index, TxBuffer, TxLength, RxBuffer,
-			&iBytesReturned);
+		return_value = SecurePINVerify(reader_index, TxBuffer, TxLength,
+			RxBuffer, &iBytesReturned);
+		*pdwBytesReturned = iBytesReturned;
+	}
+
+	/* Modify a PIN, plain CCID */
+	if (IOCTL_FEATURE_MODIFY_PIN_DIRECT == dwControlCode)
+	{
+		unsigned int iBytesReturned;
+
+		iBytesReturned = RxLength;
+		return_value = SecurePINModify(reader_index, TxBuffer, TxLength,
+			RxBuffer, &iBytesReturned);
 		*pdwBytesReturned = iBytesReturned;
 	}
 
@@ -887,6 +968,7 @@ RESPONSECODE IFDHICCPresence(DWORD Lun)
 	int oldLogLevel;
 	int reader_index;
 	_ccid_descriptor *ccid_descriptor;
+	unsigned int oldReadTimeout;
 
 	DEBUG_PERIODIC2("lun: %X", Lun);
 
@@ -895,6 +977,12 @@ RESPONSECODE IFDHICCPresence(DWORD Lun)
 
 	ccid_descriptor = get_ccid_descriptor(reader_index);
 
+	/* save the current read timeout computed from card capabilities */
+	oldReadTimeout = ccid_descriptor->readTimeout;
+
+	/* use default timeout since the reader may not be present anymore */
+	ccid_descriptor->readTimeout = DEFAULT_COM_READ_TIMEOUT;
+
 	/* if DEBUG_LEVEL_PERIODIC is not set we remove DEBUG_LEVEL_COMM */
 	oldLogLevel = LogLevel;
 	if (! (LogLevel & DEBUG_LEVEL_PERIODIC))
@@ -902,6 +990,9 @@ RESPONSECODE IFDHICCPresence(DWORD Lun)
 
 	return_value = CmdGetSlotStatus(reader_index, pcbuffer);
 
+	/* set back the old timeout */
+	ccid_descriptor->readTimeout = oldReadTimeout;
+
 	/* set back the old LogLevel */
 	LogLevel = oldLogLevel;
 
@@ -930,7 +1021,8 @@ RESPONSECODE IFDHICCPresence(DWORD Lun)
 	}
 
 	/* SCR331-DI contactless reader */
-	if ((SCR331DI == ccid_descriptor->readerID)
+	if (((SCR331DI == ccid_descriptor->readerID)
+		|| (SCR331DINTTCOM == ccid_descriptor->readerID))
 		&& (ccid_descriptor->bCurrentSlotIndex > 0))
 	{
 		unsigned char cmd[] = { 0x11 };
@@ -995,8 +1087,7 @@ void init_driver(void)
 		LogLevel = strtoul(keyValue, NULL, 0);
 
 		/* print the log level used */
-		debug_msg("%s:%d:%s LogLevel: 0x%.4X", __FILE__, __LINE__, __FUNCTION__,
-			LogLevel);
+		DEBUG_INFO2("LogLevel: 0x%.4X", LogLevel);
 	}
 
 	/* Driver options */
@@ -1006,8 +1097,7 @@ void init_driver(void)
 		DriverOptions = strtoul(keyValue, NULL, 0);
 
 		/* print the log level used */
-		debug_msg("%s:%d:%s DriverOptions: 0x%.4X", __FILE__, __LINE__,
-			__FUNCTION__, DriverOptions);
+		DEBUG_INFO2("DriverOptions: 0x%.4X", DriverOptions);
 	}
 
 	/* initialise the Lun to reader_index mapping */
@@ -1016,3 +1106,201 @@ void init_driver(void)
 	DebugInitialized = TRUE;
 } /* init_driver */
 
+
+void extra_egt(ATR_t *atr, _ccid_descriptor *ccid_desc, DWORD Protocol)
+{
+	/* This function use an EGT value for cards who comply with followings
+	 * criterias:
+	 * - TA1 > 11
+	 * - current EGT = 0x00 or 0xFF
+	 * - T=0 or (T=1 and CWI >= 2)
+	 *
+	 * Without this larger EGT some non ISO 7816-3 smart cards may not
+	 * communicate with the reader.
+	 *
+	 * This modification is harmless, the reader will just be less restrictive
+	 */
+
+	unsigned int card_baudrate;
+	unsigned int default_baudrate;
+	double f, d;
+	int i;
+
+	/* if TA1 not present */
+	if (! atr->ib[0][ATR_INTERFACE_BYTE_TA].present)
+		return;
+
+	ATR_GetParameter(atr, ATR_PARAMETER_D, &d);
+	ATR_GetParameter(atr, ATR_PARAMETER_F, &f);
+
+	/* Baudrate = f x D/F */
+	card_baudrate = (unsigned int) (1000 * ccid_desc->dwDefaultClock * d / f);
+
+	default_baudrate = (unsigned int) (1000 * ccid_desc->dwDefaultClock
+		* ATR_DEFAULT_D / ATR_DEFAULT_F);
+
+	/* TA1 > 11? */
+	if (card_baudrate <= default_baudrate)
+		return;
+
+	/* Current EGT = 0 or FF? */
+	if (atr->ib[0][ATR_INTERFACE_BYTE_TC].present &&
+		((0x00 == atr->ib[0][ATR_INTERFACE_BYTE_TC].value) ||
+		(0xFF == atr->ib[0][ATR_INTERFACE_BYTE_TC].value)))
+	{
+		if (SCARD_PROTOCOL_T0 == Protocol)
+		{
+			/* Init TC1 */
+			atr->ib[0][ATR_INTERFACE_BYTE_TC].present = TRUE;
+			atr->ib[0][ATR_INTERFACE_BYTE_TC].value = 2;
+			DEBUG_INFO("Extra EGT patch applied");
+		}
+
+		if (SCARD_PROTOCOL_T1 == Protocol)
+		{
+			/* TBi (i>2) present? BWI/CWI */
+			for (i=2; i<ATR_MAX_PROTOCOLS; i++)
+			{
+				/* CWI >= 2 ? */
+				if (atr->ib[i][ATR_INTERFACE_BYTE_TB].present && 
+					((atr->ib[i][ATR_INTERFACE_BYTE_TB].value & 0x0F) >= 2))
+				{
+					/* Init TC1 */
+					atr->ib[0][ATR_INTERFACE_BYTE_TC].present = TRUE;
+					atr->ib[0][ATR_INTERFACE_BYTE_TC].value = 2;
+					DEBUG_INFO("Extra EGT patch applied");
+
+					/* only the first TBi (i>2) must be used */
+					break;
+				}
+			}
+		}
+	}
+} /* extra_egt */
+
+
+static char find_baud_rate(unsigned int baudrate, unsigned int *list)
+{
+	int i;
+
+	DEBUG_COMM2("Card baud rate: %d", baudrate);
+
+	/* Does the reader support the annonced smart card data speed? */
+	for (i=0;; i++)
+	{
+		/* end of array marker */
+		if (0 == list[i])
+			break;
+
+		DEBUG_COMM2("Reader can do: %d", list[i]);
+
+		/* We must take into account that the card_baudrate integral value
+		 * is an approximative result, computed from the d/f float result.
+		 */
+		if ((baudrate < list[i] + 2) && (baudrate > list[i] - 2))
+			return TRUE;
+	}
+
+	return FALSE;
+} /* find_baud_rate */
+
+
+static unsigned int T0_card_timeout(double f, double d, int TC1, int TC2,
+	int clock_frequency)
+{
+	unsigned int timeout = DEFAULT_COM_READ_TIMEOUT;
+	double EGT, WWT;
+	unsigned int t;
+
+	/* Timeout applied on ISO_IN or ISO_OUT card exchange
+	 * we choose the maximum computed value.
+	 *
+	 * ISO_IN timeout is the sum of:
+	 * Terminal:					Smart card:
+	 * 5 bytes header cmd  ->
+	 *                    <-		Procedure byte
+	 * 256 data bytes	   ->
+	 * 					  <-		SW1-SW2
+	 * = 261 EGT       + 3 WWT     + 3 WWT
+	 *
+	 * ISO_OUT Timeout is the sum of:
+	 * Terminal:                    Smart card:
+	 * 5 bytes header cmd  ->
+	 * 					  <-        Procedure byte + 256 data bytes + SW1-SW2
+	 * = 5 EGT          + 1 WWT     + 259 WWT			
+	 */
+
+	/* clock_frequency is in kHz so the times are in milliseconds and not
+	 * in seconds */
+
+	/* EGT */
+	/* see ch. 6.5.3 Extra Guard Time, page 12 of ISO 7816-3 */
+	EGT = 12 * f / d / clock_frequency + (f / d) * TC1 / clock_frequency;
+
+	/* card WWT */
+	/* see ch. 8.2 Character level, page 15 of ISO 7816-3 */
+	WWT = 960 * TC2 * f / clock_frequency;
+
+	/* ISO in */
+	t  = 261 * EGT + (3 + 3) * WWT;
+	/* Convert from milliseonds to seconds rouned to the upper value
+	 * use +1 instead of ceil() to round up to the nearest interger
+	 * so we can avoid a dependency on the math library */
+	t = t/1000 +1;
+	if (timeout < t)
+		timeout = t;
+
+	/* ISO out */
+	t = 5 * EGT + (1 + 259) * WWT;
+	t = t/1000 +1;
+	if (timeout < t)
+		timeout = t;
+
+	return timeout;
+} /* T0_card_timeout  */
+
+
+static unsigned int T1_card_timeout(double f, double d, int TC1,
+	int BWI, int CWI, int clock_frequency)
+{
+	double EGT, BWT, CWT, etu;
+	unsigned int timeout;
+
+	/* Timeout applied on ISO in + ISO out card exchange
+	 *
+     * Timeout is the sum of:
+	 * - ISO in delay between leading edge of the first character sent by the 
+	 *   interface device and the last one (NAD PCB LN APDU CKS) = 260 EGT,
+	 * - delay between ISO in and ISO out = BWT,
+	 * - ISO out delay between leading edge of the first character sent by the 
+	 *   card and the last one (NAD PCB LN DATAS CKS) = 260 CWT.   
+	 */
+
+	/* clock_frequency is in kHz so the times are in milliseconds and not
+	 * in seconds */
+
+	/* see ch. 6.5.2 Transmission factors F and D, page 12 of ISO 7816-3 */
+	etu = f / d / clock_frequency;
+
+	/* EGT */
+	/* see ch. 6.5.3 Extra Guard Time, page 12 of ISO 7816-3 */
+	EGT = 12 * etu + (f / d) * TC1 / clock_frequency;
+
+	/* card BWT */
+	/* see ch. 9.5.3.2 Block Waiting Time, page 20 of ISO 7816-3 */
+	BWT = 11 * etu + (1<<BWI) * 960 * 372 / clock_frequency;
+
+	/* card CWT */
+	/* see ch. 9.5.3.1 Caracter Waiting Time, page 20 of ISO 7816-3 */
+	CWT = (11 + (1<<CWI)) * etu;
+
+	timeout = 260*EGT + BWT + 260*CWT;
+
+	/* Convert from milliseonds to seconds rounded to the upper value
+	 * we use +1 instead of ceil() to round up to the nearest greater interger
+	 * so we can avoid a dependency on the math library */
+	timeout = timeout/1000 +1;
+
+	return timeout;
+} /* T1_card_timeout  */
+
