@@ -17,12 +17,15 @@
     Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA
 */
 
-/* $Id: checksum.h,v 1.1.1.3 2005/03/15 18:55:25 rousseau Exp $ */
+/* $Id: checksum.h,v 1.2 2005-11-27 20:12:19 rousseau Exp $ */
 
 #ifndef __CHECKSUM_H__
 #define __CHECKSUM_H__
 
+#include "config.h"
+#ifdef HAVE_STDINT_H
 #include <stdint.h>
+#endif
 #include <unistd.h>
 
 extern unsigned int	csum_lrc_compute(const uint8_t *, size_t, unsigned char *);
