@@ -17,13 +17,16 @@
     Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA
 */
 
-/* $Id: proto-t1.h,v 1.1.1.3 2005/03/15 18:55:25 rousseau Exp $ */
+/* $Id: proto-t1.h,v 1.2 2005-11-27 20:12:19 rousseau Exp $ */
 
 #ifndef __PROTO_T1_H__
 #define __PROTO_T1_H__
 
-#include <unistd.h>
+#include "config.h"
+#ifdef HAVE_STDINT_H
 #include <stdint.h>
+#endif
+#include <unistd.h>
 
 enum {
 	IFD_PROTOCOL_RECV_TIMEOUT = 0x0000,
