@@ -18,7 +18,7 @@
 */
 
 /*
- * $Id: parse.c,v 1.23 2005/02/25 17:26:57 rousseau Exp $
+ * $Id: parse.c 1640 2005-09-05 14:40:13Z rousseau $
  */
 
 #include <stdio.h>
@@ -205,13 +205,98 @@ static int ccid_parse_interface_descriptor(usb_dev_handle *handle,
 
 	printf("  dwDefaultClock: %.3f MHz\n", dw2i(extra, 10)/1000.0);
 	printf("  dwMaximumClock: %.3f MHz\n", dw2i(extra, 14)/1000.0);
+	printf("  bNumClockSupported: %d %s\n", extra[18],
+		extra[18] ? "" : "(will use whatever is returned)");
+	{
+		unsigned char buffer[256*sizeof(int)];  /* maximum is 256 records */
+		int n;
 
-	printf("  bNumClockSupported: 0x%02X\n", extra[18]);
+		/* See CCID 3.7.2 page 25 */
+		n = usb_control_msg(handle,
+			0xA1, /* request type */
+			0x02, /* GET CLOCK FREQUENCIES */
+			0x00, /* value */
+			usb_interface->bInterfaceNumber, /* interface */
+			(char *)buffer,
+			sizeof(buffer),
+			2 * 1000);
+
+		/* we got an error? */
+		if (n <= 0)
+			printf("   IFD does not support GET CLOCK FREQUENCIES request\n");
+		else
+			if (n % 4) 	/* not a multiple of 4 */
+				printf("   wrong size for GET CLOCK FREQUENCIES: %d\n", n);
+			else
+			{
+				int i;
+
+				/* we do not get the expected number of data rates */
+				if ((n != extra[18]*4) && extra[18])
+				{
+					printf("   Got %d clock frequencies but was expecting %d\n",
+						n/4, extra[18]);
+
+					/* we got more data than expected */
+					if (n > extra[18]*4)
+						n = extra[18]*4;
+				}
+
+				for (i=0; i<n; i+=4)
+					printf("   Support %d kHz\n", dw2i(buffer, i));
+			}
+	}
 	printf("  dwDataRate: %d bps\n", dw2i(extra, 19));
 	printf("  dwMaxDataRate: %d bps\n", dw2i(extra, 23));
-	printf("  bNumDataRatesSupported: %d\n", extra[27]);
+	printf("  bNumDataRatesSupported: %d %s\n", extra[27],
+		extra[27] ? "" : "(will use whatever is returned)");
+	{
+		unsigned char buffer[256*sizeof(int)];  /* maximum is 256 records */
+		int n;
+
+		/* See CCID 3.7.3 page 25 */
+		n = usb_control_msg(handle,
+			0xA1, /* request type */
+			0x03, /* GET DATA RATES */
+			0x00, /* value */
+			usb_interface->bInterfaceNumber, /* interface */
+			(char *)buffer,
+			sizeof(buffer),
+			2 * 1000);
+
+		/* we got an error? */
+		if (n <= 0)
+			printf("   IFD does not support GET_DATA_RATES request\n");
+		else
+			if (n % 4) 	/* not a multiple of 4 */
+				printf("   wrong size for GET_DATA_RATES: %d\n", n);
+			else
+			{
+				int i;
+
+				/* we do not get the expected number of data rates */
+				if ((n != extra[27]*4) && extra[27])
+				{
+					printf("   Got %d data rates but was expecting %d\n", n/4,
+						extra[27]);
+
+					/* we got more data than expected */
+					if (n > extra[27]*4)
+						n = extra[27]*4;
+				}
+
+				for (i=0; i<n; i+=4)
+					printf("   Support %d bps\n", dw2i(buffer, i));
+			}
+	}
 	printf("  dwMaxIFSD: %d\n", dw2i(extra, 28));
 	printf("  dwSynchProtocols: 0x%08X\n", dw2i(extra, 32));
+	if (extra[35] & 0x01)
+			printf("   2-wire protocol\n");
+	if (extra[35] & 0x02)
+			printf("   3-wire protocol\n");
+	if (extra[35] & 0x04)
+			printf("   I2C protocol\n");
 
 	printf("  dwMechanical: 0x%08X\n", dw2i(extra, 36));
 	if (extra[36] == 0)
@@ -226,51 +311,59 @@ static int ccid_parse_interface_descriptor(usb_dev_handle *handle,
 		printf("   Card lock/unlock mechanism\n");
 
 	printf("  dwFeatures: 0x%08X\n", dw2i(extra, 40));
-	if (extra[40] == 0)
+	if (dw2i(extra, 40) == 0)
 		printf("   No special characteristics\n");
 	if (extra[40] & 0x02)
-		printf("   Automatic parameter configuration based on ATR data\n");
+		printf("   ....02 Automatic parameter configuration based on ATR data\n");
 	if (extra[40] & 0x04)
-		printf("   Automatic activation of ICC on inserting\n");
+		printf("   ....04 Automatic activation of ICC on inserting\n");
 	if (extra[40] & 0x08)
-		printf("   Automatic ICC voltage selection\n");
+		printf("   ....08 Automatic ICC voltage selection\n");
 	if (extra[40] & 0x10)
-		printf("   Automatic ICC clock frequency change according to parameters\n");
+		printf("   ....10 Automatic ICC clock frequency change according to parameters\n");
 	if (extra[40] & 0x20)
-		printf("   Automatic baud rate change according to frequency and Fi, Di parameters\n");
+		printf("   ....20 Automatic baud rate change according to frequency and Fi, Di params\n");
 	if (extra[40] & 0x40)
-		printf("   Automatic parameters negotiation made by the CCID\n");
+		printf("   ....40 Automatic parameters negotiation made by the CCID\n");
 	if (extra[40] & 0x80)
-		printf("   Automatic PPS made by the CCID\n");
+		printf("   ....80 Automatic PPS made by the CCID\n");
 	if (extra[41] & 0x01)
-		printf("   CCID can set ICC in clock stop mode\n");
+		printf("   ..01.. CCID can set ICC in clock stop mode\n");
 	if (extra[41] & 0x02)
-		printf("   NAD value other than 00 accepted (T=1)\n");
+		printf("   ..02.. NAD value other than 00 accepted (T=1)\n");
 	if (extra[41] & 0x04)
-		printf("   Automatic IFSD exchange as first exchange (T=1)\n");
+		printf("   ..04.. Automatic IFSD exchange as first exchange (T=1)\n");
 	switch (extra[42] & 0x07)
 	{
 		case 0x00:
-			printf("   Character level exchange\n");
+			printf("   00.... Character level exchange\n");
 			break;
 
 		case 0x01:
-			printf("   TPDU level exchange\n");
+			printf("   01.... TPDU level exchange\n");
 			break;
 
 		case 0x02:
-			printf("   Short APDU level exchange\n");
+			printf("   02.... Short APDU level exchange\n");
 			break;
 
 		case 0x04:
-			printf("   Short and Extended APDU level exchange\n");
+			printf("   04.... Short and Extended APDU level exchange\n");
 			break;
 	}
 
 	printf("  dwMaxCCIDMessageLength: %d bytes\n", dw2i(extra, 44));
-	printf("  bClassGetResponse: %d\n", extra[48]);
-	printf("  bClassEnveloppe: %d\n", extra[49]);
+	printf("  bClassGetResponse: 0x%02X\n", extra[48]);
+	if (0xFF == extra[48])
+		printf("   echoes the APDU class\n");
+	printf("  bClassEnveloppe: 0x%02X\n", extra[49]);
+	if (0xFF == extra[49])
+		printf("   echoes the APDU class\n");
 	printf("  wLcdLayout: 0x%04X\n", (extra[51] << 8)+extra[50]);
+	if (extra[50])
+		printf("   %d lines\n", extra[50]);
+	if (extra[51])
+		printf("   %d characters per line\n", extra[51]);
 	printf("  bPINSupport: 0x%02X\n", extra[52]);
 	if (extra[52] & 0x01)
 		printf("   PIN Verification supported\n");
